#!/bin/bash
#
# Script to configure LTSP lts.conf file with a grafical interface.
#
# Author: Jordi "Txor" Casas Ríos (jcasas@opentrends.net), openTrends Solucions i Sistemes, S.L.
#

################
# Variables
#

ZENITY=$(which zenity)
GAWK=$(which gawk)
CONFIG_FILE=/srv/tftpboot/KIWI/lts.conf
TEMP_FILE=`mktemp`
MAC_LIST=""
MAIN_TITLE="Configuració de l'ample de banda dels terminals LTSP"
MAIN_TEXT="A continuació podreu canviar el consum d'ample de banda dels terminals lleugers de l'LTSP. Podeu canviar la configuració per a tots els terminals o podeu elegir quins terminals han de reduir el seu consum (per MAC).\nQuin tipus de configuració voleu?"
MAIN_OK="Tots"
MAIN_KO="Alguns"
GLOBAL_TITLE="Configuració global."
GLOBAL_TEXT="Voleu que l'ample de banda de tots els terminals lleugers sigui reduït?"
INFO_TITLE="Selector d'adreces MAC"
INFO_TEXT="Voleu introduir alguna adreça MAC?"
MAC_TITLE="Afegir adreça MAC"
MAC_TEXT="Introduïu una adreça MAC correcte."
LIST_TITLE="Llistat d'adreces MAC"
LIST_TEXT="Voleu afegir una adreça MAC?"
APPLY_TITLE="Aplicar configuració"
APPLY_TEXT="Voleu aplicar la nova configuració?"
OK_TITLE="Configuració aplicada"
OK_TEXT="La configuració s'ha escrit correctament"
CANCEL_TITLE="Cancel·lat"
CANCEL_TEXT="Heu cancel·lat la configuració."
LIST_CONFIG_TITLE="Nova configuració"

#################
# Functions
#

# get_mac -  Shows an entry text to allow the user to enter a MAC address, reiterating in case that the MAC were invalid.
#            Print the MAC and Return 0 on success, returns 1 otherwise.
get_mac () {
	MAC_REGEXPR="([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}"
	until echo $MAC | gawk --re-interval "/$MAC_REGEXPR/ {exit 0} {exit 1}" ; do
		MAC=$($ZENITY --title="$MAC_TITLE" --text="$MAC_TEXT" --entry)
		if [ $? -eq 1 ] ; then
			return 1
		fi
	done
	echo $MAC
	return 0
}

# read_config - Read the untouchable part of the config file.
#               Prints the result and returns 0 on success, returns 1 otherwise.
read_config () {
	if [ -f $CONFIG_FILE ] ; then
		cat $CONFIG_FILE | $GAWK '! /.*\[..:..:..:..:..:..\].*/ && ! /.*LDM_DIRECTX.*/ && ! /.*X_COLOR_DEPTH.*/ {print $0}'
		return 0
	else
		return 1
	fi
}

# get_mac_list - Ask the user to fill up the list with the desired MAC addresses.
#                When finished prints the list.
get_mac_list () {
	while $ZENITY --title="$LIST_TITLE" --text="${LIST_TEXT}\n${MAC_LIST}" --question ; do
		MAC_TMP=$(get_mac)
		if [ ! -z $MAC_TMP ] ; then
			MAC_LIST="${MAC_LIST}\n${MAC_TMP}"
		fi
	done
	echo -e $MAC_LIST
}

###############
# Main code
#

$ZENITY --title="$MAIN_TITLE" --text="$MAIN_TEXT" --question --ok-label="$MAIN_OK" --cancel-label="$MAIN_KO"
if [ $? -eq 0 ] ; then
	# Global config.
	$ZENITY --title="$GLOBAL_TITLE" --text="$GLOBAL_TEXT" --question
	if [ $? -eq 0 ] ; then
		# Reduce bandwith.
		if read_config > $TEMP_FILE ; then
			echo -e "\tLDM_DIRECTX = True" >> $TEMP_FILE
			echo -e "\tX_COLOR_DEPTH = 16" >> $TEMP_FILE
		else
			$ZENITY --error --text="$CONFIG_FILE no trobat!" --timeout=10
			rm $TEMP_FILE
			exit 1
		fi
	else
		# Normal bandwith.
		if ! read_config > $TEMP_FILE ; then
			$ZENITY --error --text="$CONFIG_FILE no trobat!" --timeout=10
			rm $TEMP_FILE
			exit 1
		fi
	fi
else
	# Per MAC configuration.
	if read_config > $TEMP_FILE ; then
		# Ask for MACs.
		for i in `get_mac_list` ; do
			echo -e "[$i]" >> $TEMP_FILE
			echo -e "\tLDM_DIRECTX = True" >> $TEMP_FILE
			echo -e "\tX_COLOR_DEPTH = 16" >> $TEMP_FILE
		done
	else
		$ZENITY --error --text="$CONFIG_FILE no trobat!" --timeout=10
		rm $TEMP_FILE
		exit 1
	fi
fi
# Review changes and write config.
$ZENITY --title="$LIST_CONFIG_TITLE" --text-info --filename=$TEMP_FILE
if $ZENITY --title="$APPLY_TITLE" --text="$APPLY_TEXT" --question ; then
	cat $TEMP_FILE > $CONFIG_FILE && $ZENITY --title="$OK_TITLE" --text="$OK_TEXT al fitxer $CONFIG_FILE." --info
else
	$ZENITY --title="$CANCEL_TITLE" --text="$CANCEL_TEXT" --error --timeout=10
fi
rm $TEMP_FILE
exit 0

